%option header-file="lex.yy.h"
%option noyywrap

%x COMMENT DIRECTIVE

letter [a-zA-z]
digit [0-9]
id {letter}({letter}|{digit}|"_")*
newLine \n|\r|(\r\n)
directive "#"(include|define)
keyword (auto|break|case|char|continue|do|default|const|double|else|enum|extern|for|if|goto|float|int|long|register|return|signed|static|sizeof|short|struct|switch|typedef|union|void|while|volatile|unsigned)
numericConstant {digit}{digit}*
op ("+"|"-"|"/"|"*"|"<"|">"|"=="|">="|"<="|"=")
string \"(\\.|[^"\\])*\"
multiLineComment "/*"
endComment "*/"

%top{
  /* This goes at the top of the generated file */
  #include "html.h"
}

%%
<INITIAL>{
  {multiLineComment} {
    BEGIN(COMMENT);
    open_tag(C_CYAN);
    text_print(yytext);
  }
  {keyword} {
    open_tag(C_LBLUE);
    text_print(yytext);
    close_tag();
  }
  {numericConstant} {
    open_tag(C_LRED);
    text_print(yytext);
    close_tag();
  }
  {string} {
    open_tag(C_RED);
    text_print(yytext);
    close_tag();
  }
  {directive} {
    BEGIN(DIRECTIVE);
    open_tag(C_LGREEN);
    text_print(yytext);
  }
  {newLine} {
    text_print(yytext);
  }
  {op} {
    open_tag(C_BLUE);
    text_print(yytext);
    close_tag();
  }
  {id} {
    open_tag(C_GREEN);
    text_print(yytext);
    close_tag();
  }

}

<COMMENT>{
  {endComment} {
  BEGIN(INITIAL);
  text_print(yytext);
  close_tag();
  }
  . {
    text_print(yytext);
  }
  {newLine} {
    text_print(yytext);
  }
}

<DIRECTIVE>{
  {newLine} {
  BEGIN(INITIAL);
  text_print(yytext);
  close_tag();
  }
  . {
    text_print(yytext);
  }
}
